import java.io.Serializable;
import java.util.Collection;
import java.util.Random;
import java.util.UUID;

import com.hazelcast.core.Hazelcast;
import com.hazelcast.core.IMap;
import com.hazelcast.query.EntryObject;
import com.hazelcast.query.Predicate;
import com.hazelcast.query.PredicateBuilder;

public class Demo {

	private static final String MAP_NAME = "blobs";
	private static final String SPECIAL = UUID.randomUUID().toString();

	public static void main(String[] args) throws Exception {

		IMap<String, Blob> blobs = Hazelcast.getMap(MAP_NAME);
		
		if (args.length > 0 && args[0].equalsIgnoreCase("server")) {
			
			// Add the "special" blob to the map.  
			// This is the one we'll be querying for.
			System.out.println("Initializing Map");
			blobs.put(SPECIAL, new Blob(SPECIAL));
		
			// Add some more blobs to the map
			// just to give Hazelcast something to do.
			for (int i = 0; i < 10; i++) {
				String name = UUID.randomUUID().toString();
				blobs.put(name, new Blob(name));
			}
			
			long then = System.currentTimeMillis();
			long start = then;
			int returnCode = 0;
			Random random = new Random();
			while(System.currentTimeMillis() - start < 5000) {
				long now = System.currentTimeMillis();
				
				//Print a character every second, just to show we're alive.
				if (now - then > 1000) {
					System.out.print("s");
					then = now;
				}

				
				//Retrieve the special blob in the usual way 
				//and update its lastUpdateTime.
				Blob b = blobs.get(SPECIAL);
				b.setLastUpdateTime(now);
				blobs.put(SPECIAL,b);
				
				Thread.sleep(random.nextInt(100) + 1);

				Blob getBlob = blobs.get(SPECIAL);
				long getTime = getBlob.getLastUpdateTime();
				
				//Retrieve the special blob via a query and 
				//verify that the lastUpdateTime is what it should be.
				
				EntryObject e = new PredicateBuilder().getEntryObject();
				Predicate<?,?> predicate = e.get("name").equal(SPECIAL);
				Collection<Blob> theBlobs = blobs.values(predicate);
				Blob queryBlob = theBlobs.iterator().next();
				long queryTime = queryBlob.getLastUpdateTime();

				if (queryTime != getTime) {
					System.out.println("ERROR! Expected " + getTime + " Got " + queryTime);
					returnCode = 1;
				}
			}
			System.out.println("Done");
			System.exit(returnCode);
			
		} else {
			while (true) {
				Thread.sleep(1000);
				System.out.print("c");
			}
		}
	}
}

class Blob implements Serializable {
	private static final long serialVersionUID = 1L;
	private String name;
	private long lastUpdateTime;
	
	public Blob(String name) {
		this.name = name;
		this.lastUpdateTime = System.currentTimeMillis();
	}
	
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public long getLastUpdateTime() {
		return lastUpdateTime;
	}
	public void setLastUpdateTime(long lastUpdateTime) {
		this.lastUpdateTime = lastUpdateTime;
	}
}
